// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
//-----------------------------------------------------------------------------
//
//						XMM Science Analysis System (SAS)
//						(c) 1998 European Space Agency
//
//-----------------------------------------------------------------------------
//
//	NaN--- Implements the NotANumber class that represents the ieee NaN.
//


#include <Tracer.h>


#include "NAN.h"

NotANumber NaN;


/* set data item to "illegal" */

#define FNANMASK   0x7F800001 /* mask bits 1 - 8; all set on NaNs */
                              /* all 0 on underflow  or 0. */

#define DNANMASK   0x7FF00000 /* mask bits 1 - 11; all set on NaNs */
                              /* all 0 on underflow  or 0. */

#define NANMASK    1

/* set data item to "illegal" */
#define IS_E_NULL(a) (*(unsigned *)&(a) == FNANMASK)
#define SET_E_NULL(a) (*(unsigned *)&(a) = FNANMASK)

#ifdef WORDS_BIGENDIAN
#define IS_D_NULL(a) (*(unsigned *)&(a) == DNANMASK) && (*((unsigned *)&(a)+1) == NANMASK);
#define SET_D_NULL(a) *(unsigned *)&(a) = DNANMASK, *((unsigned *)&(a)+1) = NANMASK;
#else
#define IS_D_NULL(a) (*((unsigned *)&(a) + 1) == DNANMASK) && (*(unsigned *)&(a) == NANMASK);
#define SET_D_NULL(a) *((unsigned *)&(a) + 1) = DNANMASK, *(unsigned *)&(a) = NANMASK;
#endif



void
NotANumber::set( float& f) const
{
	PROC
	SET_E_NULL(f);
	ENDPROC
}


void
NotANumber::set( double& d) const
{
	PROC
	SET_D_NULL(d);
	ENDPROC
}

bool
NotANumber::isSet( const float& f) const
{
	PROC
	RETURN IS_E_NULL( f );
}


bool
NotANumber::isSet( const double& d) const
{
	PROC
	RETURN IS_D_NULL( d );
}


