// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.

#ifndef __VECTOR_H
#define __VECTOR_H


template<class T>
class Vec
{
	public :
		Vec();
		~Vec();
		Vec( const Vec & );
		Vec & operator=( const Vec & );
		void push_back( const T & item );
		unsigned int size() const;
		const T & operator[](unsigned int index ) const;
	private :
		vector<T *>	_data;
};


// ******************************
// Inlined functions beyond here.
// ******************************


template<class T>
inline Vec<T>::Vec()
{
}


template<class T>
inline Vec<T>::~Vec()
{
	for(unsigned int i = 0; i < _data.size(); ++i ) delete _data[ i ];
}


template<class T>
inline
Vec<T>::Vec( const Vec & value )
{
	for( unsigned int i = 0; i < value.size(); ++i )
	{
		T * newItem = new T;
		*newItem = *value._data[ i ];
		_data.push_back( newItem );
	} // for
}


template<class T>
inline Vec<T>&
Vec<T>::operator=( const Vec & value )
{
	for( unsigned int i = 0; i < _data.size(); ++i ) delete _data[ i ];
	for( unsigned int i = 0; i < value.size(); ++i )
	{
		T * newItem = new T;
		*newItem = *value._data[ i ];
		_data.push_back( newItem );
	} // for

	return *this;
}


template<class T>
inline void Vec<T>::push_back( const T & item )
{
	T * newItem = new T;
	*newItem = item;
	_data.push_back( newItem );
}


template<class T>
inline unsigned int Vec<T>::size() const
{
	return _data.size();
}

template<class T>
inline const T & Vec<T>::operator[]( unsigned int index ) const
{
	return *_data[ index ];
}



#endif


