// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
/* Revision History
 *
 * $Id: binary_tree.h,v 1.7 2008/09/26 16:38:55 eojero Exp $
 *
 */


/**************************************************************************/
/*                                                                        */
/*  FILENAME                                                              */
/*  ========                                                              */
/*  binary_tree.h                                                         */
/*                                                                        */
/*                                                                        */
/*  SUBSYSTEM                                                             */
/*  =========                                                             */
/*  XMM SAS - UTILITIES                                                   */
/*                                                                        */
/*                                                                        */
/*  TITLE                                                                 */
/*  =====                                                                 */
/*  Public header file for interface to BinaryTree type.                  */
/*                                                                        */
/*                                                                        */
/*  AUTHORS                                                               */
/*  =======                                                               */
/*  Mark Thomas                                                           */
/*  CARA Software & Services Ltd                                          */
/*                                                                        */
/*                                                                        */
/*  CONSTRAINTS                                                           */
/*  ===========                                                           */
/*  All typedefs and functions given in this header file are for          */
/*  public use.                                                           */
/*                                                                        */
/*                                                                        */
/*  DESCRIPTION                                                           */
/*  ===========                                                           */
/*  Public Header file for interface to BinaryTree type.                  */
/*                                                                        */
/*                                                                        */
/*  EXAMPLE                                                               */
/*  =======                                                               */
/*                                                                        */
/*  main()                                                                */
/*  {                                                                     */
/*  	BinaryTreeType		Tree_Of_Tokens;                           */
/*                                                                        */
/*                                                                        */
/*  }                                                                     */
/*                                                                        */
/*                                                                        */
/**************************************************************************/


#ifndef __BINARY_TREE_H
#define __BINARY_TREE_H


template<class T>
class BinaryTree
{

	public :

		// Constructors
		BinaryTree();
		BinaryTree& operator=( const BinaryTree & );
		BinaryTree( const BinaryTree & );
		BinaryTree( const T & nodeValue );
		BinaryTree& operator=( const T & nodeValue );
		~BinaryTree();

		void setNode( const T & token );
		const T & node() const;
		void setRight( const BinaryTree & tree );
		const BinaryTree & right() const;
		void setLeft( const BinaryTree & tree );
		const BinaryTree & left() const;

	private :

		T * 			_value;
		BinaryTree<T> *		_left;
		BinaryTree<T> * 	_right;

};


// *****************************
// Inlined functions beyond here
// *****************************

template<class T>
inline BinaryTree<T>::BinaryTree()
{

	

	_left = 0;
	_right = 0;
	_value = new T;

	

}


template<class T>
inline BinaryTree<T>::~BinaryTree()
{
	

	delete _left;
	delete _right;
	delete _value;

	

}


template<class T>
inline void
BinaryTree<T>::setNode( const T & node )
{

	

	*_value = node;

	

}


template<class T>
inline const T &
BinaryTree<T>::node() const
{
	
	return *_value;

}


template<class T>
inline void
BinaryTree<T>::setRight( const BinaryTree & tree )
{
	

	delete _right;

	_right = new BinaryTree();
	*_right = tree;

	
}


template<class T>
inline const BinaryTree<T> &
BinaryTree<T>::right() const
{
	
	return *_right;

}


template<class T>
inline void
BinaryTree<T>::setLeft( const BinaryTree & tree )
{
	

	delete _left;

	_left = new BinaryTree();
	*_left = tree;

	

}


template<class T>
inline const BinaryTree<T> &
BinaryTree<T>::left() const
{
	
	return *_left;

}


template<class T>
inline BinaryTree<T>&
BinaryTree<T>::operator=( const BinaryTree<T> & tree )
{
	

	*_value = *tree._value;
	delete _left;
	_left = 0;
	delete _right;
	_right = 0;

	if( tree._left != static_cast<const BinaryTree<T> * >( 0 ) )
	{
		_left = new BinaryTree;
		*_left = *tree._left;

	} /* if */

	if( tree._right != static_cast<const BinaryTree<T> * >( 0 ) )
	{
		_right = new BinaryTree;
		*_right = *tree._right;

	} /* if */

	return *this;

}


template<class T>
inline BinaryTree<T>::BinaryTree( const BinaryTree<T> & tree )
{

	

	_value = new T;
	*_value = *tree._value;

	if( tree._left != static_cast<const BinaryTree<T> *>( 0 ) )
	{
		_left = new BinaryTree;
		*_left = *tree._left;

	}
	else
	{
		_left = 0;

	} /* if */

	if( tree._right != static_cast<const BinaryTree<T> *>( 0 ) )
	{
		_right = new BinaryTree;
		*_right = *tree._right;

	}
	else
	{
		_right = 0;

	} /* if */


	

}


template<class T>
inline BinaryTree<T>::BinaryTree( const T & nodeValue )
{
	
	_left = 0;
	_right = 0;
	_value = new T;
	*_value = nodeValue;
	
}


template<class T>
inline BinaryTree<T>&
BinaryTree<T>::operator=( const T & nodeValue )
{
	
	delete _left;
	_left = 0;
	delete _right;
	_right = 0;
	*_value = nodeValue;
	return *this;
}


#endif

