! ESA (C) 2000-2018 
!
! This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
!
!    SAS is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    SAS is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
!
!	stimetestf90.cc --- test class STime from f90
!
!	DESCRIPTION:
!		Intended to the correctness of F90 I/F to class STime
!
!	INVOCATION:
!		stimetest
!
!	RETURNS:
!		0	 : all tests passed successfullly
!		!=0	 : at least one test failed; corresponding message printed stderr
!
!	CHANGE HISTORY:
!		UL, 2001-07-11: 1.0.0: - first version
!
!	AUTHOR:
!		U. Lammers, Wed Jul 11 11:03:47 MET DST 2001
!
!	$Header: /sasbuild/repositories/sasdev/sas/packages/utils/test/stimetestf90.f90,v 1.4 2008/09/26 16:38:55 eojero Exp $
!  
program stimetest
use STime

    implicit none
    type(STimeHandleT)  :: t1, t2, t3, t4, t5, t6, t51, t7, t8, t9, t10, &
                           t11, t12, t13, t14, t15, t16, t17, t18, t22, &
                           t23, t25, t24, t21, t20
    type(JulianDayHandleT) :: jdh
    integer             :: n
    character(len=23)   :: t2string, t5string, t51string, t8string, t8string2,&
                           t10string, t12string, t13string, t22str
    character(len=24)   :: t4string
    real(kind=double)   :: t5julday, t6mjulday, t8offset, t9mt8, t, td, &
                           TWO_PI, sidBerlin, mins, secs, hours, sidTime, diff
    integer             :: ih,im, is

	! default construction - object should be initialized to
	! STime::earliest
	t1 = ST_new()
    if (ST_isNotSameInstant(t1, ST_earliest())) call stfail(1);

	! construction from year/month/day/hour/min/sec tuple
    t2string = '1997-10-23T14:15:34.123'
    t2 = ST_new(1997, 10, 23, 14, 15, 34, .123_double)
    if (trim(ST_toString(t2, 3, ST_FITS)) /= t2string) call stfail(2);

	! construction from string object
    t3 = ST_new(t2string)
    if (trim(ST_toString(t3, 3, ST_FITS)) /= t2string) call stfail(3)
    if (trim(ST_toString(t3, 0)) /= t2string(1:len(t2string)-4)) call stfail(4)

    do n=1, 3
        if (trim(ST_toString(t3, n)) /= t2string(1:len(t2string)-3+n)) &
            call stfail(5)
    enddo

    t4string = 'Wed Oct 29 15:37:19 1997'
	t4 = ST_new(t4string)
    if (trim(ST_toString(t4, 0, ST_CALENDAR)) /= t4string) call stfail(6)

	! construction from MJD/JD/DYN number
	! conversion to MJD/JD number
	t5string = '1997-10-13T00:00:00.000'
	t5julday = 2450734.5_double
    t5 = ST_new(t5julday, ST_JD)
    if (trim(ST_toString(t5)) /= t5string) call stfail(7)
    if (ST_julianDayFull(ST_julianDay(t5)) /= t5julday) call stfail(8)

	t6mjulday = t5julday - 2400000.5
    t6 = ST_new(t6mjulday, ST_MJD)
    if (trim(ST_toString(t6)) /= t5string) call stfail(9)
    jdh = ST_modifiedJulianDay(t6)
    if (ST_julianDayFull(jdh) /= t6mjulday) &
        call stfail(10)
    call ST_release(jdh)

    t51 = ST_new(1998.5_double, ST_DYN)
    t51string = '1998-07-02T12:00:00.000'
    if (trim(ST_toString(t51)) /= t51string) call stfail(11)

	! copy constructor
    t7 = ST_clone(t5);
    if (trim(ST_toString(t7)) /= t5string) call stfail(12)

	! assignment
    call ST_release(t7)
    call ST_assign(t7, t2);
    if (trim(ST_toString(t7)) /= t2string) call stfail(12)

	! +=, -=
    t8string = '1999-12-31T23:59:00.100'
	t8string2 = '2000-01-01T00:04:03.100'
    t8 = ST_new(t8string)
	t8offset = 5. * ST_secondsPerMinute() + 3.
    call ST_addTo(t8, t8offset)
    if (trim(ST_toString(t8)) /= t8string2) call stfail(13)
    call ST_subtractFrom(t8, t8offset)
    if (trim(ST_toString(t8)) /= t8string) call stfail(14)

	! -, +, differnce between time instances
    t9 = ST_add(t8, t8offset)
    if (trim(ST_toString(t9)) /= t8string2) call stfail(15)
	t9mt8 = ST_diff(t9, t8)
    if (int(t9mt8 * 1000000. +.5) /= int(t8offset * 1000000. + .5)) &
        call stfail(16)

    t10 = ST_sub(t9, 10. * ST_secondsPerDay())  	! t9 - 10 days
	t10string = '1999-12-22T00:04:03.100'
    if (trim(ST_toString(t10)) /= t10string) call stfail(17)

	! binary operators
	t11 = ST_new()
    call ST_assign(t11, t10)
    if (.not. ST_isSameInstant(t11, t10)) call stfail(18)

    call ST_subtractFrom(t11, 0.001_double)
    if (.not. ST_isEarlierThan(t11, t10)) call stfail(19)
    if (.not. ST_isEarlierThanOrSameInstant(t11, t10)) call stfail(20)
    if (.not. ST_isNotSameInstant(t11, t10)) call stfail(21)

    call ST_addTo(t11, 0.002_double)
    if (.not. ST_isLaterThan(t11, t10)) call stfail(22)
    if (.not. ST_isLaterThanOrSameInstant(t11, t10)) call stfail(23)
    if (.not. ST_isNotSameInstant(t11, t10)) call stfail(24)

    t12string = '1997-12-01T08:30:00.000'
    t12 = ST_new(t12string)
    if (trim(ST_toString(t12)) /= t12string) call stfail(25)

	!
	!	check if leap seconds are correctly handled
	!
	t13string = '1995-12-31T23:59:59.100'
    t13 = ST_new(t13string)
    if (trim(ST_toString(t13)) /= t13string) call stfail(26)

    call ST_addTo(t13, 1._double) ! within a leap second
    if (trim(ST_toString(t13)) /= '1995-12-31T23:59:60.100') call stfail(27)

	call ST_addTo(t13, 1._double)	!	after the leap seond
    if (trim(ST_toString(t13)) /= '1996-01-01T00:00:00.100') call stfail(28)
    if (ST_leapSeconds(t13) /= 30) call stfail(29)

    t14 = ST_new('1994-01-01')
    t15 = ST_new('1997-06-30T23:59:59.1')

	diff = ST_diff(t15, t14)
    if (.not. (diff >= 110332801.1_double .and. &
               diff <= 110332801.100001_double)) call stfail(30)
	diff = ST_diff(ST_add(t15, 1._double), t14)
    if (.not. (diff >= 110332802.1_double .and. &
               diff <= 110332802.100001_double)) call stfail(31) ! during leap
	diff = ST_diff(ST_add(t15, 2._double), t14)
	if (.not. (diff >= 110332803.1_double .and. &
               diff <= 110332803.100001_double)) call stfail(32) !	after leap

    t16 = ST_new('1998-01-01')
	diff = ST_diff(t16, t14) - ((4 * 365 + 1) * 86400._double + &
        ST_leapSeconds(t16) - ST_leapSeconds(t14))
    if (.not. abs(diff) <= 1.d-6) call stfail(33)

	!	time differences across leap seconds
    t17 = ST_new('1988-01-01')
    t18 = ST_new('1987-12-31T23:59:59')
    t = -2.;
    do while (t<0.)
        td = ST_diff(t18, t17)
        if (.not. (td < 0. .and. abs(td/t-1.) <= 1.e-6)) call stfail(34)
        t = t + .1
        call ST_addTo(t18, .1_double)
    enddo

	!	a few precision related tests
    t22str = '1999-12-31T23:59:59.999'
    t22 = ST_new('1999-12-31T23:59:59.999')
    if (trim(ST_toString(t22)) /= t22str) call stfail(35)
    t23 = ST_new('1999-12-31T23:59:59.99999')
    if (trim(ST_toString(t22)) /= t22str) call stfail(36)

    t25 = ST_new('2001-01-01T00:00:59.999999999')
    if (trim(ST_toString(t25, 5)) /= '2001-01-01T00:00:59.99999') &
        call stfail(37)
    t24 = ST_new('1999-12-31T23:59:59.999999999')
    if (trim(ST_toString(t24, 6)) /= '1999-12-31T23:59:59.999999') &
        call stfail(38)

    write(*, *) 'This should be the current GMT ->', &
        trim(ST_toString(ST_currentTime()))
    write(*, *) 'Again                          ->', &
        trim(ST_toString(ST_new('now')))
    write(*, *) 'Local time is                  ->', &
		trim(ST_toString(ST_currentTime(.false.)))
    if (.not. (ST_isSameInstant(ST_currentTime(), ST_new("now")) .or. &
               ST_isSameInstant(ST_currentTime(), ST_new("now")) .or. &
               ST_isSameInstant(ST_currentTime(), ST_new("now")))) &
        call stfail(39)

    if (.not. ST_isValidTime('1999-01-01T00:00:00.123')) call stfail(40)
    if (.not. ST_isValidTime('   1999-01-01T00:00:00.123    ')) call stfail(41)
	if (ST_isValidTime('1999-01-01T')) call stfail(42)
	if (ST_isValidTime('XXXXXXXX')) call stfail(43)
    if (ST_isValidTime('1999-01-01T00:00:61')) call stfail(44)
    if (ST_isValidTime('             ')) call stfail(45)

    if (ST_isValidTime(-1._double)) call stfail(46)

    if (.not. ST_isValidTime(2000, 2, 29)) call stfail(47)
    if (ST_isValidTime(1900, 2, 29)) call stfail(48)

    if (ST_isValidTime(1999, 8, 8, 24, 0, 0, 0._double)) call stfail(49)

    TWO_PI = 2.*acos(-1.);
	!
	!	sidereal time in Berlin (longitude=+13.3deg) at 1982-04-15T20:00 UTC
	!	is 10:27:48
	!	see "Fundamental Astronomy", p. 42
	!
    t20 = ST_new('1982-04-15T20:00:00')
    sidBerlin = ST_siderealTime(t20, 13.3_double*TWO_PI/360._double)
	secs = sidBerlin/TWO_PI*86400._double
    mins = floor(secs/60._double)
    secs = mod(secs, 60._double)
    hours = floor(mins/60._double)
	mins = mod(mins, 60._double)
    ih = INT(hours+.5)
    im = INT(mins+.5)
	is = INT(secs+.5)
    write(*, *) 'Sidereal time in Berlin on ', trim(ST_toString(t20)), &
        ' GMT is: ', ih, ':', im, ':', is
    if (.not. (ih == 10 .and. im==27 .and. is==48)) call stfail(50)

    t21 = ST_new('2000-07-08T09:44:30')
    sidTime = ST_siderealTime(t21, -80.38216389*TWO_PI/360.)
	write(*, '(A, F12.6)') 'Sidereal time: ', 360.*sidTime/TWO_PI

    call ST_release(t1)
    call ST_release(t2)
    call ST_release(t3)
    call ST_release(t4)
    call ST_release(t5)
    call ST_release(t6)
    call ST_release(t51)
    call ST_release(t8)
    call ST_release(t9)
    call ST_release(t10)
    call ST_release(t11)
    call ST_release(t12)
    call ST_release(t13)
    call ST_release(t14)
    call ST_release(t15)
    call ST_release(t16)
    call ST_release(t17)
    call ST_release(t18)
    call ST_release(t22)
    call ST_release(t23)
    call ST_release(t25)
    call ST_release(t24)
    call ST_release(t21)
    call ST_release(t20)

	write(*, *) 'All tests sucessfully passed'
end program

subroutine stfail(code)
    integer, intent(in)  :: code
    write(*, *) 'STime test #', code, 'failed'
    stop
end subroutine
