// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
#include <stringutil.h>
#include "common.h"
#include <cassert>

int main()
{
	assert(trim("       ") == "");
	assert(trim("test ")			== "test" );
	assert(trim(" test ")			== " test" );
	assert(trim("")			== "" );
	assert(trim(" test ",true)            == "test");

	assert(unquote(" test ")		== " test " );
	assert(unquote("")			== "" );
	assert(unquote("'test'")		== "test" );
	assert(unquote("\"test\"")		== "test" );
	assert(unquote("'test")		== "'test" );
	assert(unquote("  'test'  ")		== "test" );
	assert(unquote("  ' test '   ")	== " test " );

	assert(lowercase(" HeLlO!1@ ")	== " hello!1@ " );

	assert(uppercase(" HeLlO!1@ ")	== " HELLO!1@ " );

	std::vector<std::string> v(breakup(",,,   : \t\t\t\n "));
	assert(!v.size());

	v = breakup("string");
	assert(v.size() == 1 && v[0]=="string");

	v = breakup("   ,,::x::y::|||12  ||2", " ,:|");
	assert(v.size()==4 && v[0]=="x" && v[1]=="y" && v[2]=="12" && v[3]=="2");

	v = quotedBreakup("", ":");
	assert(v.size()==0);

	v = quotedBreakup(":", ":");
	assert(v.size()==0);

	v = quotedBreakup("\"", ":");
	assert(v.size()==1 && v[0]=="\"");

	v = quotedBreakup("s1", ":");
	assert(v.size()==1 && v[0]=="s1");

	v = quotedBreakup("s1\"::\"x", ":");
	assert(v.size()==1 && v[0]=="s1\"::\"x");

	v = quotedBreakup("\"x::\":s1", ":");
	assert(v.size()==2 && v[0]=="\"x::\"" && v[1]=="s1");

	v = quotedBreakup("t=\"1 2\" x=1", " ");
	assert(v.size()==2 && v[0]=="t=\"1 2\"" && v[1]=="x=1");

	v = quotedBreakup("circle 348.5 359 44 text=\"yyy / zzz\" include=0", " ");
	assert(v.size()==6 && v[0]=="circle" && v[1]=="348.5" &&
		   v[2]=="359" && v[3]=="44" && v[4]=="text=\"yyy / zzz\"" &&
		   v[5]=="include=0");

	v = quotedBreakup("circle(1, 2, 3) text=\"yyy / zzz\" text={ok babe}",
					  " =", "\"({");
	assert(v.size()==5 && v[0]=="circle(1, 2, 3)" && v[1]=="text" &&
		   v[2]=="\"yyy / zzz\"" && v[3]=="text" && v[4]=="{ok babe}");

	assert(toNumber<int>("12334") == 12334 &&
		   toNumber<double>("3.1415") == 3.1415);

	assert(replaceall("X1XX", "X", "XX") == "XX1XXXX");
	assert(replaceall("I do not believe in this", "do not ", "") ==
		   "I believe in this");
	assert(replaceall("bla bli", "x", "yy") == "bla bli");
	assert(replaceall("polygon(xx)||polygon(yy)", "polygon(", "polygon2(") ==
			"polygon2(xx)||polygon2(yy)");
	assert(replaceall("XX1XXXX", "XX", "X") == "X1XX");

	const char *names[] = { "/etc/passwd", "/", "", ".", "./", "./bla.txt",
						    "bla.txt", "bla.txt:TAB%ATT" },
			   *dn[] = { "/etc", "/", "", ".", ".", ".", ".", "." },
			   *bn[] = { "passwd", "", "", "", "", "bla.txt", "bla.txt",
						 "bla.txt" };

	for (unsigned n=0; n<NUM_OF(names); ++n) {
		assert(Filename::split(names[n]).first == dn[n]);
		assert(Filename::split(names[n]).second == bn[n]);
	}

	return 0;
}
